﻿using System;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;

namespace HeuristicLab.Clients.Hive.CloudManager.Views {
  public partial class AddCertificate : Form {
    private BackgroundWorker worker = new BackgroundWorker();
    public bool ErrorOccured { get; set; }

    public AddCertificate() {
      InitializeComponent();
      worker.DoWork += new DoWorkEventHandler(AddCertificateTask);
      worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(WorkerCompleted);
    }

    private void btnBrowseCertificateFile_Click(object sender, EventArgs e) {
      OpenFileDialog ofd = new OpenFileDialog();
      ofd.Filter = "All Certificate Files|*.pfx";
      if (ofd.ShowDialog() == DialogResult.OK) {
        tbCertificateFile.Text = ofd.FileName;
      }
    }

    private void btnCancel_Click(object sender, EventArgs e) {
      this.Close();
    }

    private void btnOk_Click(object sender, EventArgs e) {
      if (tbCertificateFile.Text.Length == 0) {
        MessageBox.Show("Certificate file is required", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      } else if (tbCertificatePassword.Text.Length == 0) {
        MessageBox.Show("Password is required", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      } else {
        SetControlsEnabled(false);
        progressBar.Visible = true;
        worker.RunWorkerAsync();
      }

    }

    private void SetControlsEnabled(bool isEnabled) {
      tbCertificatePassword.Enabled = isEnabled;
      btnBrowseCertificateFile.Enabled = isEnabled;
      btnCancel.Enabled = isEnabled;
      btnOk.Enabled = isEnabled;
    }

    private void AddCertificateTask(object sender, DoWorkEventArgs e) {
      try {
        // simulate service call
        Thread.Sleep(3000);
      }
      catch (Exception ex) {
        ErrorOccured = true;
        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void WorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
      progressBar.Visible = false;
      if (!ErrorOccured) {
        this.Close();
      } else {
        this.Show();
        SetControlsEnabled(true);
      }
    }
  }
}
